from .utils import *


def translation_quantitative_scoring(original_dataframe_path, translated_dataframe_path):
    """
    This function takes two dataframes as input and prints out the translation quality scores.
    The scores are calculated by comparing the original text to the translated text.
    The scores are a float between 0 and 100.

    Args:
        original_dataframe (dataframe): The original dataframe.
        translated_dataframe (dataframe): The translated dataframe.

    Returns:
        dataframe: The dataframe with the translation quality scores.
    """
    # read all files that follow the pattern as pd dataframe
    original_dataframe = pd_read_pattern(original_dataframe_path)
    translated_dataframe = pd_read_pattern(translated_dataframe_path)
    total_characters_to_translate = 0
    sum_words_translated = 0
    sum_words_original = 0
    sum_characters_translated = 0
    sum_characters_original = 0
    for column in original_dataframe.columns:
        original_dataframe = original_dataframe.astype({column: str})
        translated_dataframe = translated_dataframe.astype({column: str})
        list_score_words = []
        list_score_characters = []
        list_score_completeness = []
        completeness_score = 0
        scores_words = 0
        scores_character = 0
        for row in range(len(original_dataframe)):
            if (translated_dataframe[column][row] in ["", np.nan, "nan"]) & (
                original_dataframe[column][row] == translated_dataframe[column][row]
            ):
                pass
            elif (translated_dataframe[column][row] in ["", np.nan, "nan"]) & (
                original_dataframe[column][row] != translated_dataframe[column][row]
            ):
                completeness_score += 1
            else:
                scores = translation_quality_scores(
                    original_dataframe[column][row], translated_dataframe[column][row]
                )
                scores_character += scores[0]
                scores_words += scores[1]
                total_characters_to_translate += scores[2]
                sum_words_translated += scores[3]
                sum_words_original += scores[4]
                sum_characters_translated += scores[5]
                sum_characters_original += scores[6]
        list_score_completeness.append(
            1 - completeness_score / (len(translated_dataframe))
        )
        list_score_words.append(scores_words / (len(translated_dataframe)))
        list_score_characters.append(scores_character / (len(translated_dataframe)))
    print(
        "Percentage of words translated vs available:",
        "{:.2f}".format(sum(list_score_words) / len(list_score_words) * 100), "%"
    )
    print(
        "Percentage of characters translated vs available:",
        "{:.2f}".format(sum(list_score_characters) / len(list_score_characters) * 100), "%"
    )
    print(
        "Percentage of non-null translations:",
        "{:.2f}".format(
            sum(list_score_completeness) / len(list_score_completeness) * 100), "%"
        )
    
    print(
        "Total number of characters to be translated:",
        int(total_characters_to_translate),
    )
    print(
        "Total number of words translated/available: " +
        str(sum_words_translated) + "/" + str(sum_words_original)
    )
    print(
        "Total number of characters translated/available: "+
        str(sum_characters_translated) + "/" + str(sum_characters_original)
    )

    results_dict = {'words_percentage': "{:.2f}".format(sum(list_score_words) / len(list_score_words) * 100) + '%', 
            'characters_percentage': "{:.2f}".format(sum(list_score_characters) / len(list_score_characters) * 100) + '%',
            'non_null_percentage': "{:.2f}".format(
            sum(list_score_completeness) / len(list_score_completeness) * 100) + '%',
            'total_characters': int(total_characters_to_translate),
            'total_words_translated/available': str(sum_words_translated) + "/" + str(sum_words_original),
            'total_characters_translated/available': str(sum_characters_translated) + "/" + str(sum_characters_original),}
    return results_dict



from nltk.tokenize import word_tokenize
from collections import Counter
import numpy as np
import pandas as pd
import glob

def pd_read_pattern(pattern):
    files = glob.glob(pattern)

    df = pd.DataFrame()
    for f in files:
        df = df.append(pd.read_csv(f))

    return df.reset_index(drop=True)

df = pd_read_pattern('somefile*.csv')

def remove_punc(string):
    punc = """!()-[]{};:'"\, <>./?@#$%^&*_~1234567890"""
    for ele in string:
        if ele in punc:
            string = string.replace(ele, "")
    return string


def diff(text1, text2):
    text1 = text1.replace(" ", "")
    text2 = text2.replace(" ", "")
    return sum(c1 != c2 for c1, c2 in zip(text1, text2)) / len(text1)


def translation_quality_scores(original_text, translated_text):
    """
    This function takes two strings as input and returns three translation quality score.
    The score is calculated by comparing the original text to the translated text.
    The score is a float between 0 and 1.

    Args:
          original_text (str): The original text.
          translated_text (str): The translated text.

      Returns:
          characters_translated_vs_available: The percentage of characters translated vs characters available.
          words_translated_vs_available: The percentage of words translated vs words available.
          characters_to_translate: The number of characters to be translated.
    """
    # tokenize the strings
    original_token = word_tokenize(original_text)
    translated_token = word_tokenize(translated_text)
    # remove punctuation
    original_token = [remove_punc(i) for i in original_token]
    # remove empty strings
    original_token = list(filter(None, original_token))
    # lowercase
    original_token = [x.lower() for x in original_token]
    translated_token = [x.lower() for x in translated_token]
    # list of translated words
    translated_words = list(
        (Counter(original_token) - Counter(translated_token)).elements()
    )
    # calculate the scores

    characters_translated_vs_available = sum([len(i) for i in translated_words]) / sum(
        [len(i) for i in original_token]
    )

    sum_words_translated = len(translated_words)

    sum_words_original = len(original_token)

    sum_characters_translated = sum([len(i) for i in translated_words])

    sum_characters_original = sum([len(i) for i in original_token])

    words_translated_vs_available = len(translated_words) / len(original_token)

    characters_to_translate = sum(len(x) for x in original_text.split())

    return (
        characters_translated_vs_available,
        words_translated_vs_available,
        characters_to_translate,
        sum_words_translated,
        sum_words_original,
        sum_characters_translated,
        sum_characters_original
    )



import pandas as pd
from src.dataframetranslationscoring.scoring import translation_quantitative_scoring

df_original = pd.read_excel("./tests/SAIB_TablesNames_Translated.xlsx")
df_translated = df_original[["Microsoft translation"]]
df_original = df_original[["Portuguese table name"]]
df_translated.rename(
    columns={"Microsoft translation": "Portuguese table name"}, inplace=True
)
df_original = df_original[:5]
df_translated = df_translated[:5]
print(translation_quantitative_scoring(df_original, df_translated))



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()
setuptools.setup(
    name="dataframetranslationscoring",  # Replace with your own username
    version="0.0.6",
    author="Nabil Berjaoui",
    author_email="nabil.berjaoui@adm.com",
    packages=setuptools.find_packages("src"),
    package_dir={"": "src"},
    description="A translation quality package",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://dev.azure.com/ADM-Production-CorpIT1/Incremental%20Strategic%20Delivery/_git/ISD_Translation",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
)
